<?php
/*
	HTTP fetching/parsing tools.
*/

/* Получает страницу, разрешая $maxredirect редиректов.
	CURL умеет это делать и сам, но при определённых настройках сервера редиректы включить нельзя.
	В таком случае выполняется их отслеживание вручную */
function curl_exec_follow($ch, /*int*/ &$maxredirect = null) { 
	$mr = $maxredirect === null ? 5 : intval($maxredirect);
	if (ini_get('open_basedir') == '' && ini_get('safe_mode' == 'Off')) {
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, $mr > 0);
		curl_setopt($ch, CURLOPT_MAXREDIRS, $mr);
	} else {
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
		if ($mr > 0) {
			$newurl = curl_getinfo($ch, CURLINFO_EFFECTIVE_URL);

			$rch = curl_copy_handle($ch);
			curl_setopt($rch, CURLOPT_HEADER, true);
			curl_setopt($rch, CURLOPT_NOBODY, true);
			curl_setopt($rch, CURLOPT_FORBID_REUSE, false);
			curl_setopt($rch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($rch, CURLOPT_HEADERFUNCTION, 'curl_no_callback');

			do {
				curl_setopt($rch, CURLOPT_URL, $newurl);
				$header = curl_exec($rch);
				if (curl_errno($rch)) {
					$code = 0;
				} else {
					$code = curl_getinfo($rch, CURLINFO_HTTP_CODE);
					if ($code == 301 || $code == 302) {
						preg_match('/Location:(.*?)\n/', $header, $matches);
						$newurl = trim(array_pop($matches));
					} else {
						$code = 0;
					}
				}
			} while ($code && --$mr);

			curl_close($rch);

			if (!$mr) {
				if ($maxredirect === null) {
					trigger_error('Too many redirects. When following redirects, libcurl hit the maximum amount.', E_USER_WARNING);
				} else {
					$maxredirect = 0;
				}
				return false;
			}

			curl_setopt($ch, CURLOPT_URL, $newurl);
		}
	}
	return curl_exec($ch);
}
/* Blyadish CURL does not allow to unset callback, only to change it.
	So we need an empty callback to unset anything users might have set. */
function curl_no_callback($ch, $data) {
	return binlen($data);
}

function curl_fetch($addr) {
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $addr);
	curl_setopt($ch, CURLOPT_HEADER, 0);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data=curl_exec_follow($ch);
	if (curl_errno($ch)!=0)
		die('Query error: '.curl_errno($ch));
	$rcode=curl_getinfo($ch, CURLINFO_HTTP_CODE);
	curl_close($ch);
	$ret[0]=$rcode;
	$ret[1]=$data;
	return $ret;
}

/* Blyadish mb_strlen breaks everything when it replaces strlen, so we have to call it explicitly */
function binlen(&$data) {
	if (function_exists('mb_strlen'))
		return mb_strlen($data, '8bit');
	else /* sa-a-afe! */
		return strlen($data);
}

/* Скачивает файлы со всякими опциями.
  Не используйте из нескольких потоков сразу или для второго скачивания. */
class curlDownloader {
	public $ch;
	public $header='';
	public $data='';
	
	/* Настройки передачи */
	public $maxReceive=null;	/* null=нет ограничения */
	public $maxRedirects=null; /* null=по умолчанию */
	
	/* Флаги состояния */
	private $dataSize=0;
	private $headerParsed=false;

	public $aborted=false;
	public $abortMessage='';
	
	function __construct($source) {
		$this->ch = curl_init();
		curl_setopt($this->ch, CURLOPT_URL, $source);
		curl_setopt($this->ch, CURLOPT_HEADER, FALSE);
		curl_setopt($this->ch, CURLOPT_HEADERFUNCTION, array($this, 'curl_header'));
		curl_setopt($this->ch, CURLOPT_WRITEFUNCTION, array($this, 'curl_post_data'));
		curl_setopt($this->ch, CURLINFO_HEADER_OUT, true);
	}
	
	function __destruct() {
		curl_close($this->ch);
	}
	
	/* Получает данные. Возвращает true, если не был прерван по ошибке (aborted) */
	function fetch() {
		curl_exec_follow($this->ch, $this->maxRedirects);
		return (!$this->aborted);
	}
	
	/* Прервать закачку навсегда */
	function abort($msg='') {
		$this->aborted=true;
		$this->abortMessage=$msg;
		/* Любая из функций приёма данных теперь вернёт ноль. */
	}
	
	/*
		Мы должны ограничить размер скачиваемых данных.
		HTTP может сообщить размер оставшихся данных в заголовке, тогда мы прерываемся сразу же.
		Иначе прерываемся по достижении максимума.
	*/

	function curl_header($ch, $data) {
		if ($this->aborted) return 0;

		$sz=binlen($data);
		/* Всегда проверяем, что maxReceive не нарушен, даже в заголовке */
		if (($this->maxReceive!==null) && ($this->dataSize + $sz > $this->maxReceive)) {
			$this->abort('Data size exceeded.');
			return 0;
		}

		/* Пишем в заголовок */
		$this->header.=$data;
		$this->dataSize+=$sz;
		return $sz;
	}

	function curl_post_data($ch, $data) {
		if ($this->aborted) return 0;

		/* Обрабатывем заголовок */
		if (!$this->headerParsed) {
			$this->parseHeader();
			if ($this->aborted) return 0;
		}

		$sz=binlen($data);
		/* Проверяем, что maxReceive не нарушен */
		if (($this->maxReceive!==null) && ($this->dataSize + $sz > $this->maxReceive)) {
			$this->abort('Data size exceeded.');
			return 0;
		}

		/* Пишем данные */
		$this->data.=$data;
		$this->dataSize+=$sz;
		return $sz;
	}
	
	/* Обрабатывает полученный полностью заголовок. Должна быть вызвана до начала обработки данных. */
	private function parseHeader() {
		/* Не выходите из этой функции посередине, в конце взводится флаг */
		$contentLength=curl_getinfo($this->ch, CURLINFO_CONTENT_LENGTH_DOWNLOAD);
		if ($contentLength!==FALSE) {
			if (($this->maxReceive!==null) && ($this->dataSize + $contentLength > $this->maxReceive))
				$this->abort('Data size exceeded.');
		}
		$this->headerParsed=true;
	}
}

?>